package persistence;

import org.junit.Before;
import org.junit.BeforeClass;
import org.junit.Test;
import persistence.category.CategoryEntity;

import java.io.File;
import java.io.IOException;

import static org.junit.Assert.assertEquals;
import static org.junit.jupiter.api.Assertions.*;

class GenericCSVDAOTest {

    GenericCSVDAO<CategoryEntity> sut;

    public void beforeEachDirty() throws IOException {
        File file = new File("test.csv");

        if (file.exists()){
            file.delete();
        }
        file.createNewFile();

        this.sut = new GenericCSVDAO<CategoryEntity>(file,CategoryEntity::new);


    }

    @Test
    public void addWorks() throws IOException {

        beforeEachDirty();


        assertEquals(0, sut.getALl().size());

        var entityToAdd = new CategoryEntity("categoryName",99);
        sut.add(entityToAdd);
        var foundEntity = sut.getALl().stream().findFirst().orElseThrow();

        assertEquals(entityToAdd,foundEntity);




    }

    @Test
    public void removeWorks() throws IOException {

        beforeEachDirty();


        var entityToAdd = new CategoryEntity("categoryName",99);
        sut.add(entityToAdd);

        sut.remove(entityToAdd);

        assertEquals(0, sut.getALl().size());
    }

    @Test
    public void removeAllWorks() throws IOException {

        beforeEachDirty();


        var entityToAdd1 = new CategoryEntity("categoryName1",101);
        sut.add(entityToAdd1);
        var entityToAdd2 = new CategoryEntity("categoryName2",102);
        sut.add(entityToAdd2);
        assertEquals(2, sut.getALl().size());
        sut.removeAll();

        assertEquals(0, sut.getALl().size());
    }
}